#ifndef CAMOBJECT_H
#define CAMOBJECT_H

#include <QString>

#include "waittingthread.h"
#include "./sdk/inc/TUCamApi.h"

const int PID_MICHROME6 = 0xEC09;
const int PID_MICHROME20 = 0xEC0B;
const int PID_MICHROME5PRO = 0xEC07;

class CamObject
{    
public:
    ~CamObject();

    static CamObject *getInstance();
    int getCamPID() { return m_pid; }
    QString getSavedPath() { return m_savedPath; }
    HDTUCAM getCamHandle() { return  m_opCam.hIdxTUCam; }
    WaittingThread *getWaittingThread() { return &m_waittingThread; }

    void stopWaittingForFrames();
    void startWaittingForFrames();

    void setSavedPath(const QString &path) { m_savedPath = path; }
    void stopSavingImages();
    void startSavingImages(const int &totalFrames, const int &intervalTime, const int &savedFormat);

    int getSrcWidth() { return m_waittingThread.getSrcWidth(); }
    int getSrcHeight() { return m_waittingThread.getSrcHeight(); }
    bool isSaving() { return m_waittingThread.isSaving(); }
    bool isWaitting() { return m_waittingThread.isRunning(); }

    static TUCAM_INIT s_itCam;
    static CamObject *s_instance;
    static QString m_configPath;

    class GarbageCollector
    {
        public:
        ~GarbageCollector()
        {
            if (NULL != CamObject::s_instance)
            {
                delete CamObject::s_instance;
                CamObject::s_instance = NULL;
            }
        }
    };
    static GarbageCollector gc;

private:
    CamObject();

    int m_pid;
    TUCAM_OPEN m_opCam;
    WaittingThread m_waittingThread;

    QString m_savedPath;
};

#endif // CAMOBJECT_H
